/**************************************************************************
*
* Function:  initPLL_2146x -- Initialize the PLL of ADSP-2146x
*
*
*            #include <initPLL.h> 
* Prototype: void initPLL_2146x(int multi,int div,int ddr2ckr,bool indiv,int lpckr);       
*
*
* initPLL_2146x function usage
*
* S.No  parameter name	Input range/values	  Description	 
*
* 1.    multi			0 - 63        		  Chooses the multiplier value from 0 to 63. Value of 0 corresponds to 64.
* 2.    div             0 ,1,2 and 3 		  Chooses the divider value,   0 - Divider value 2
*																		   1 - Divider value 4
*																		   2 - Divider value 8
*																		   3 - Divider value 16
* 3.    ddr2ckr         0 , 2 and 4 		  Chooses the cclk to ddr2clock ratio,   0 - CCLK-DDR2CLK ratio 2
*																		             2 - CCLK-DDR2CLK ratio 3
*																		   			 4 - CCLK-DDR2CLK ratio 4																		   
*																		  
* 4.    indiv			true , false          True - Enables the Input Divider		      																		  																	   
*											  False - Disables the Input Divider		

* 5.	lckr           0 ,1,2 and 3 		  Chooses the cclk to linkportclock ratio, 
*																		  0 - CCLK-LPCLK ratio 2
*																		  1 - CCLK-LPCLK ratio 2.5
*																		  2 - CCLK-LPCLK ratio 3
*																		  3 - CCLK-LPCLK ratio 4				
*
*            if ((multi > 0) && (div > 0))
*			 {
*				if( indiv == FALSE )
*				{
*					CCLK = CLKIN * multi / div;
*				}
*				else
*				{
*					CCLK = 2 * CLKIN * multi / div;
*				}
*
*				if( DDR2CKR > 0 )
*				{
*					DDR2CLK = CCLK / ddr2ckr;	
*				}
*				if( LCKR > 0 )
*				{
*					LCLK = CCLK / lckr;	
*				}
*				
*				
*			}
*			else if( multi < 0 )
*			{
*				if( indiv == FALSE )
*				{
*					CCLK = CLKIN * multi;
*				}
*				else
*				{
*					CCLK = CLKIN * multi / 2(default divider value)
*				}
*				
*				if( DDR2CKR > 0 )
*				{
*					DDR2CLK = CCLK / ddr2ckr;	
*				}
*				if( LCKR > 0 )
*				{
*					LCLK = CCLK / lckr;	
*				}
*			}
*			else 
*			{
*				if( indiv == FALSE )
*				{
*					CCLK = 2 * CLKIN / div;
*				}
*				else
*				{
*					CCLK = CLKIN / div;
*				}
*				
*				if( DDR2CKR > 0 )
*				{
*					DDR2CLK = CCLK / ddr2ckr;	
*				}
*				if( LCKR > 0 )
*				{
*					LCLK = CCLK / lckr;	
*				}
*			}
*					
* 				
*
* (c) Copyright 2002-2004 Analog Devices, Inc.  All rights reserved.
*     $Revision: 1.5 $
**************************************************************************/

.file_attr libGroup="initPLL.h";
.file_attr libName="libc";
.file_attr libFunc="_initPLL_2146x";
.file_attr FuncName="_initPLL_2146x";
.file_attr prefersMem="internal";
.file_attr prefersMemNum="30";

#include "lib_glob.h"
#include <def21469.h>

.SEGMENT/CODE Code_Space_Name; // Sections used by the Visual DSP++ tool for the
			       // library function
.FILE RTL_FILENAME;

.GLOBAL   _initPLL_2146x_15000020_workaround;

_initPLL_2146x_15000020_workaround:
	
	r1 = dm(1,i6); 
    
	// Below code has the Workaround for 1500020 Anomaly
	
    	//Step 1 - change the PLLD to 4
	ustat1 = dm(PMCTL);
	bit clr USTAT1 PLLD16 ;
	bit set USTAT1 PLLD4 | DIVEN ;
	dm(PMCTL) = ustat1;

	//Step 2 - wait for dividers to stabilize
	lcntr=16, do first_div_delay until lce;
	first_div_delay: nop;
	
	//Step 3 - set INDIV bit now to bring down the VCO speed and enter the bypass mode
	bit clr ustat1 DIVEN;
	bit set ustat1 INDIV | PLLBP;
	dm(PMCTL) = ustat1;
	
	//Step 4 - wait for the PLL to lock
	lcntr=4096, do first_bypass_delay until lce;
	first_bypass_delay:nop;

	//Step 5 - come out of the bypass mode
	ustat1 = dm(PMCTL);
	bit clr ustat1 PLLBP;
	dm(PMCTL) = ustat1;
	
	//Step 6 - wait for dividers to stabilize
	lcntr=16, do second_div_delay until lce;
	second_div_delay:nop;

	
	//Step 7 - set the required PLLM and INDIV values here  and enter the bypass mode

	ustat1=dm(PMCTL);
	bit clr ustat1 INDIV|PLLM63;

	r0 = 0;
    	check_for_indiv:   
 		comp(r1,r0);	// Check whether the input divider is enabled.
 		if le jump extract_mult;
		bit set ustat1 INDIV;   // If input divider is enabled set INDIV bit.
		
	extract_mult:  
	    r2 = 0x3F;
	    r4 = r4 and r2; // Extract the multiplier bits
	    r2 = ustat1;
	    r2 = r2 or r4;
	    ustat1 = r2;	// set the multiplier value in the ustat1
	
	dm(PMCTL)=ustat1;
	
	//Step 8 - wait for the PLL to lock 
	lcntr=4096, do second_bypass_delay until lce;
	second_bypass_delay:nop;

	//Step 9 - come out of the bypass mode
	ustat1 = dm(PMCTL);
	bit clr ustat1 PLLBP;
	dm(PMCTL) = ustat1;
	
	//Step 10 - wait for dividers to stabilize
	lcntr=16, do third_div_delay until lce;
	third_div_delay:nop;

	ustat1=dm(PMCTL);
	bit clr ustat1 PLLD16 | 0x1C0000 | LPCKR4;
	
	check_for_ddckr:
		comp(r12,r0);		// Check for the DDCKR value.
    		r2 = 0x07;			
    		r12 = r12 and r2;	// Extract the DDCKR bits
		r12 = lshift r12 by 18; 
  		r2 = ustat1;
   		r2 = r2 or r12;
   		ustat1 = r2;	// set the sdckr bits value in the ustat1
    
        check_for_lckr:
		r1 = dm(2,i6);  // Read LCKR value from stack
		r2 = 0x03;			
		r12 = r12 and r2;	// Extract the LCKR bits
		r12 = lshift r12 by 21; 
		r2 = ustat1;
		r2 = r2 or r12;
		ustat1 = r2;	// set the lckr bits value in the ustat1    
      
	check_for_div:       
	    comp(r8,r0); // Check whether the divider is used.
	    r2 = 0x3;	
	    r8 = r8 and r2; // Extract the divisor bits
	    r8 = lshift r8 by 6;	
	    r2 = ustat1;
	    r2 = r2 or r8;
	    ustat1 = r2;	// set the divisor value in the ustat1
	    
       bit set ustat1 DIVEN;
       dm(PMCTL)=ustat1;
       
       //Step 12 - wait for the dividers to stabilize
       lcntr=16, do fourth_div_delay until lce;
       fourth_div_delay: nop;

exit_initPLL:           

	
    FETCH_RETURN // Instruction used by the Visual DSP++ tools library for
		 // returning from the library function
    RETURN (DB);
    RESTORE_STACK
    RESTORE_FRAME
    
_initPLL_2146x_15000020_workaround.end:
.ENDSEG;